{"cells": [{"cell_type": "markdown", "metadata": {}, "source": ["# Effects of Basis Set Size: The Molecular Case\n", "\n", "\n", "In this exercise you will compute the equilibrium energy of\n", "H$_2$ at the Hartree-Fock level using different basis sets, i.e. the 6-31G and 6-311G sets (with which you are familiar from last week), as well as Dunning's\n", "correlation-consistent aug-cc-pVTZ basis. Correlation-consistent basis\n", "sets were defined such that systematic improvement over total energies\n", "and molecular properties is possible. \n", "\n", "First again we again import the required modules:"]}, {"cell_type": "code", "execution_count": 2, "metadata": {}, "outputs": [], "source": ["import psi4\n", "import pandas as pd\n", "import numpy as np\n", "import matplotlib.pyplot as plt\n", "import sys \n", "\n", "sys.path.append(\"..\")\n", "from helpers import *\n", "\n", "plt.style.use(['seaborn-poster', 'seaborn-ticks'])"]}, {"cell_type": "markdown", "metadata": {}, "source": ["then we set the maximum ressources that can be used"]}, {"cell_type": "code", "execution_count": 3, "metadata": {}, "outputs": [{"name": "stdout", "output_type": "stream", "text": ["\n", "  Memory set to   1.863 GiB by Python driver.\n", "  Threads set to 4 by Python driver.\n"]}], "source": ["psi4.set_memory('2 GB')\n", "psi4.set_num_threads(4)"]}, {"cell_type": "markdown", "metadata": {}, "source": ["next we define our $H_2$ geometry, setting the H molecules at equilibrium distance of each other, i.e. 0.7414\u00c5. We are using XYZ coordinates for the `psi4.geometry`, but as you saw last week also Z-matrix geometries can be used here."]}, {"cell_type": "code", "execution_count": 4, "metadata": {}, "outputs": [], "source": ["h2 = psi4.geometry(\"\"\"\n", "0 1\n", "H 0.0 0.0 0.0\n", "H 0.0 0.0 0.7414\n", "\"\"\")\n"]}, {"cell_type": "markdown", "metadata": {}, "source": ["We can have a look at out molecule using the helper function `drawXYZ`, to double-check everything looks fine"]}, {"cell_type": "code", "execution_count": 4, "metadata": {}, "outputs": [{"data": {"application/3dmoljs_load.v0": "<div id=\"3dmolviewer_16661649220145373\"  style=\"position: relative; width: 400px; height: 400px\">\n        <p id=\"3dmolwarning_16661649220145373\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n        </div>\n<script>\n\nvar loadScriptAsync = function(uri){\n  return new Promise((resolve, reject) => {\n    var tag = document.createElement('script');\n    tag.src = uri;\n    tag.async = true;\n    tag.onload = () => {\n      resolve();\n    };\n  var firstScriptTag = document.getElementsByTagName('script')[0];\n  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n});\n};\n\nif(typeof $3Dmolpromise === 'undefined') {\n$3Dmolpromise = null;\n  $3Dmolpromise = loadScriptAsync('https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js');\n}\n\nvar viewer_16661649220145373 = null;\nvar warn = document.getElementById(\"3dmolwarning_16661649220145373\");\nif(warn) {\n    warn.parentNode.removeChild(warn);\n}\n$3Dmolpromise.then(function() {\nviewer_16661649220145373 = $3Dmol.createViewer($(\"#3dmolviewer_16661649220145373\"),{backgroundColor:\"white\"});\nviewer_16661649220145373.zoomTo();\n\tviewer_16661649220145373.addModel(\"2\\n\\n H    0.000000000000    0.000000000000   -0.370700000000\\n H    0.000000000000    0.000000000000    0.370700000000\\n\",\"xyz\");\n\tviewer_16661649220145373.setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n\tviewer_16661649220145373.zoomTo();\nviewer_16661649220145373.render();\n});\n</script>", "text/html": ["<div id=\"3dmolviewer_16661649220145373\"  style=\"position: relative; width: 400px; height: 400px\">\n", "        <p id=\"3dmolwarning_16661649220145373\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n", "        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n", "        </div>\n", "<script>\n", "\n", "var loadScriptAsync = function(uri){\n", "  return new Promise((resolve, reject) => {\n", "    var tag = document.createElement('script');\n", "    tag.src = uri;\n", "    tag.async = true;\n", "    tag.onload = () => {\n", "      resolve();\n", "    };\n", "  var firstScriptTag = document.getElementsByTagName('script')[0];\n", "  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n", "});\n", "};\n", "\n", "if(typeof $3Dmolpromise === 'undefined') {\n", "$3Dmolpromise = null;\n", "  $3Dmolpromise = loadScriptAsync('https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js');\n", "}\n", "\n", "var viewer_16661649220145373 = null;\n", "var warn = document.getElementById(\"3dmolwarning_16661649220145373\");\n", "if(warn) {\n", "    warn.parentNode.removeChild(warn);\n", "}\n", "$3Dmolpromise.then(function() {\n", "viewer_16661649220145373 = $3Dmol.createViewer($(\"#3dmolviewer_16661649220145373\"),{backgroundColor:\"white\"});\n", "viewer_16661649220145373.zoomTo();\n", "\tviewer_16661649220145373.addModel(\"2\\n\\n H    0.000000000000    0.000000000000   -0.370700000000\\n H    0.000000000000    0.000000000000    0.370700000000\\n\",\"xyz\");\n", "\tviewer_16661649220145373.setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n", "\tviewer_16661649220145373.zoomTo();\n", "viewer_16661649220145373.render();\n", "});\n", "</script>"]}, "metadata": {}, "output_type": "display_data"}], "source": ["drawXYZ(h2)"]}, {"cell_type": "markdown", "metadata": {}, "source": ["Edit the code cell below to compute the energy (using `psi4.energy()` funciton) for the different basis sets selected"]}, {"cell_type": "code", "execution_count": null, "metadata": {}, "outputs": [], "source": ["basissets = ['6-31G', '6-311G', 'aug-cc-pvtz'] # these are the basis sets we are going to use\n", "\n", "psi4.set_options({'reference':'UHF'}) # We are using Unrestriced Hartee Fock\n", "\n", "for basis in basissets:\n", "    psi4.core.set_output_file(f'{basis}-output.log', False) # save in seperate log files\n", "    E = None    # EDIT HERE\n", "                # we want to perform a single point energy calculation once per basis set\n", "\n", "    print(basis, E)"]}, {"cell_type": "markdown", "metadata": {}, "source": ["```{admonition} Exercise 1\n", ":class: exercise\n", "Include a table of the the calculated energies using the three different basis sets\n", "```"]}, {"cell_type": "markdown", "metadata": {}, "source": ["Lets have a look at one of the outputs (e.g for the `6-31G` basis). We can use the shell command `!grep -options SEARCHTERM FILENAME` for this. (Remember: The `!` denotes that this is not a python command but a bash program.) \n", "\n", "As the SCF procedure is iterative we can search for the word `iter` in the output. By including `-A 1 -B 3` we get 1 line after the last find and 3 lines before the first find, which contain some helpful context to make sense of the results. "]}, {"cell_type": "code", "execution_count": null, "metadata": {}, "outputs": [], "source": ["!grep -A 1 -B 3 iter 6-31G-output.log"]}, {"cell_type": "markdown", "metadata": {}, "source": ["```{admonition} Exercise 2\n", ":class: exercise\n", "What is the meaning of the Delta E column? What is DIIS? (Hint: to get more details, it may help to check the [HF page on psi4 manual](https://psicode.org/psi4manual/master/scf.html))\n", "```"]}, {"cell_type": "markdown", "metadata": {}, "source": ["Next, let's plot the SCF convergence for the three tested basis sets. You can use a little helper function called `psi4_read_scf` which will read in the table shown above and return it to you as pandas dataframe that you can easily plot. "]}, {"cell_type": "code", "execution_count": null, "metadata": {}, "outputs": [], "source": ["# Read in scf table like so:\n", "\n", "# Syntax of psi4_read_scf function:\n", "nameOfDataFrame = psi4_read_scf('nameOfOutputFile.log')\n", "\n", "# to view the table you can simply type the name of the DataFrame, i.e.:\n", "nameOfDataFrame"]}, {"cell_type": "markdown", "metadata": {}, "source": ["For the plotting we will use matplotlib. We will also include in the plot the exact value. As explained in the introduction, this was obtained with a post-Hartree Fock calculation, and its value for the H$_2$ molecule is $E_{exact} = -1.174474{}$ a.u.\n", "\n", "In particular, we suggest you to produce two plots, one with the full set of points, and the other one excluding the first SCF iteration. This can help you to appreciate the differences between the three basis sets. In the cell code below, we already set both subplots. Edit the code in order to plot the results for the three basis sets."]}, {"cell_type": "code", "execution_count": null, "metadata": {}, "outputs": [], "source": ["# Subplot 1: full data and exact value reference\n", "fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(14,7)) # create figure with two subplots\n", "fig.tight_layout(pad=7.0) # add spacing between supblots \n", "\n", "# we use the table index and the Total Energy column for plotting\n", "# repeat the following command once per basis set to have all on the same plot\n", "ax1.plot(nameOfDataFrame.index, nameOfDataFrame['Total Energy'], '.-', label='label') \n", "\n", "ax1.set_xlabel('SCF Iteration') # set axis labels and title\n", "ax1.set_ylabel('E [a.u]')\n", "ax1.set_title('SCF convergence')\n", "\n", "ax1.axhline(-1.174474, label='Exact value', linestyle='--', color='gray') # horizontal line for the exact energy\n", "\n", "ax1.legend() # show legend\n", "\n", "# Subplot 2: excluding first SCF point to focus on the difference along SCF cycles\n", "# you can exclud the first point of an array just by [1:]. This will select all point from 1 (so excluding 0) to the end\n", "# example: array DATA --> DATA[1:]\n", "# repeat the following command once per basis set to have all on the same plot\n", "ax2.plot(nameOfDataFrame.index[1:], nameOfDataFrame['Total Energy'][1:], '.-', label='label') \n", "\n", "ax2.set_xlabel('SCF Iteration') # set axis labels\n", "ax2.set_ylabel('E [a.u]')\n", "ax2.set_title('SCF convergence - restricted range')\n", "\n", "ax2.legend() # show legend\n", "plt.show()"]}, {"cell_type": "markdown", "metadata": {}, "source": ["```{admonition} Exercise 3\n", ":class: exercise\n", "How does the basis set size influence the SCF convergence behaviour? Include in your report the plot of the energy convergence along SCF iterations. \n", "```"]}], "metadata": {"celltoolbar": "Tags", "kernelspec": {"display_name": "Computational Chemistry", "language": "python", "name": "compchem"}, "language_info": {"codemirror_mode": {"name": "ipython", "version": 3}, "file_extension": ".py", "mimetype": "text/x-python", "name": "python", "nbconvert_exporter": "python", "pygments_lexer": "ipython3", "version": "3.8.12"}}, "nbformat": 4, "nbformat_minor": 4}